/*************************************************************************
Title:    testing output to a HD44780 based LCD display.
Author:   Peter Fleury  <pfleury@gmx.ch>  http://jump.to/fleury
File:     $Id: test_lcd.c,v 1.6 2004/12/10 13:53:59 peter Exp $
Software: AVR-GCC 3.3
Hardware: HD44780 compatible LCD text display
          ATS90S8515/ATmega if memory-mapped LCD interface is used
          any AVR with 7 free I/O pins if 4-bit IO port mode is used
**************************************************************************/
#include <stdlib.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include "lcd_1.h"

#define F_CPU 16000000UL  // Clock speed = 16MHz
#define XTAL 16000000

#define delay(us)  _delayFourCycles( ( ( 1*(XTAL/4000) )*us)/1000 )

/*
** constant definitions
*/
static const PROGMEM unsigned char copyRightChar[] =
{
	0x07, 0x08, 0x13, 0x14, 0x14, 0x13, 0x08, 0x07,
	0x00, 0x10, 0x08, 0x08, 0x08, 0x08, 0x10, 0x00
};


/*
** function prototypes
*/ 
void wait_until_key_pressed(void);


void wait_until_key_pressed(void)
{
    unsigned char temp1, temp2;
    unsigned int i;
    
    do {
        temp1 = PIND;                  // read input
        for(i=0;i<65535;i++);
        temp2 = PIND;                  // read input
        temp1 = (temp1 & temp2);       // debounce input
    } while ( temp1 & _BV(PIND2) );
    
    loop_until_bit_is_set(PIND,PIND2);            /* wait until key is released */
}

void delay_loop(int i)
{
   while(i)
   {
      delay(10000);
      i--;
   }
}

void move_disp_left(unsigned int count)
{
         while(count+1)
		 {
	      lcd_command(2,LCD_MOVE_DISP_LEFT);
          delay_loop(40);
		  count--;
         }
}


int main(void)
{
    char buffer[7];
    int  num=134;
    unsigned char i;

    /* initialize display, cursor off */
    lcd_init(1,LCD_DISP_ON);  //INIT Display0, upper part
	if (LCD_NO_OF_DISP == 2)
	{
	   lcd_init(2,LCD_DISP_ON);  //INIT Display1, lower part
    }

    for (;;) {                           /* loop forever */
        /* 
         * Test 1:  write text to display
         */
        
        /* clear display and home cursor */
        lcd_clrscr(1);
		if (LCD_NO_OF_DISP == 2)
		{
		    lcd_clrscr(2);
        }

        /* put string to display (line 1) with linefeed */
        lcd_puts(1,"LCD Test Line 1, upper part\n");
		lcd_puts(1,"LCD and Line 2, upper part");

        /* wait until push button PD2 (INT0) is pressed */
        wait_until_key_pressed();
        lcd_clrscr(1);     /* clear display home cursor */

        /* move cursor to position 8 on line 2 */
        lcd_gotoxy(1,1,1);  
        
        /* write single char to display */
        lcd_putc(1,':');
        
        /* now write something to display part2 */
		if (LCD_NO_OF_DISP == 2)
		{
		   lcd_puts(2,"LCD Line1, lower part");
		   lcd_gotoxy(2,1,2);
		   lcd_puts(2,"LCD Line2, lower part");
		}


		//PORTD = 0x20;
        /* wait until push button PD2 (INT0) is pressed */
        wait_until_key_pressed();
        
        
        /*
         * Test 2: use lcd_command() to turn on cursor
         */
        
        /* turn on cursor */
        lcd_command(1,LCD_DISP_ON_CURSOR);

        /* put string */
        lcd_puts(1,"CurOn");
        
        /* wait until push button PD2 (INT0) is pressed */
        wait_until_key_pressed();

		/* turn off cursor */
        lcd_command(1,LCD_DISP_ON);

        /* wait until push button PD2 (INT0) is pressed */
        wait_until_key_pressed();


        /*
         * Test 3: display shift
         */
        
        lcd_clrscr(1);     /* clear display home cursor */
        if (LCD_NO_OF_DISP == 2)
		{
           lcd_clrscr(2);
		}
        /* put string from program memory to display */
        lcd_puts_P(1, "Line 1 longer than 14 characters\n");
        lcd_puts_P(1, "Line 2 longer than 14 characters");
        
		/* wait until push button PD2 (INT0) is pressed */
        wait_until_key_pressed();

        /* move BOTH lines one position to the left */
        lcd_command(1,LCD_MOVE_DISP_LEFT);
		
        /* wait until push button PD2 (INT0) is pressed */
        wait_until_key_pressed();

        
        /*
         *   Test: Display integer values
         */
        
        lcd_clrscr(1);   /* clear display home cursor */
        
        /* convert interger into string */
        itoa( num , buffer, 10);
        
        /* put converted string to display */
        lcd_puts(1,buffer);
        
        /* wait until push button PD2 (INT0) is pressed */
        wait_until_key_pressed();
        
        
        /*
         *  Test: Display userdefined characters
         */

       lcd_clrscr(1);   /* clear display home cursor */
       
       lcd_puts(1,"Copyright: ");

       /*
        * load two userdefined characters from program memory
        * into LCD controller CG RAM location 0 and 1
        */
       lcd_command(1,_BV(LCD_CGRAM));  /* set CG RAM start address 0 */
       for(i=0; i<16; i++)
       {
           lcd_data(1,pgm_read_byte_near(&copyRightChar[i]));
       }
       
       /* move cursor to position 0 on line 2 */
       /* Note: this switched back to DD RAM adresses */
       lcd_gotoxy(1,1,1);
       
       /* display user defined (c), built using two user defined chars */
       lcd_putc(1,0);
       lcd_putc(1,1);

	   while(1==1)
	   {
	   lcd_clrscr(2);   /* clear display home cursor */
	   lcd_puts(2,"Das ist ein ganz langer Text, der auch nt, der auch nicht auf zwei Zeilen Displa");
	   delay_loop(200);
       move_disp_left(12);
	   lcd_clrscr(2);   /* clear display home cursor */
	   lcd_puts(2,"ganz langer Text, der auch nicht auf zwenicht auf zwei Zeilen Display passt...  ");
       move_disp_left(12);
	   delay_loop(200);
	   }
       

       /* wait until push button PD2 (INT0) is pressed */
       wait_until_key_pressed();
              
    }
}
